@@ -32,6 +32,10 @@ if __name__ == "__main__":
         myvar = root.get_child(["0:Objects", "2:MyObject", "2:MyVariable"])
         obj = root.get_child(["0:Objects", "2:MyObject"])
         print("myvar is: ", myvar)
+        print("myobj is: ", obj)
+
+        # Stacked myvar access
+        # print("myvar is: ", root.get_children()[0].get_children()[1].get_variables()[0].get_value())
 
     finally:
         client.disconnect()
