@@ -15,6 +15,7 @@ from opcua.common.node import Node
 from opcua.common.manage_nodes import delete_nodes
 from opcua.common.subscription import Subscription
 from opcua.common import utils
+from opcua.common import ua_utils
 from opcua.crypto import security_policies
 from opcua.common.shortcuts import Shortcuts
 from opcua.common.structures import load_type_definitions, load_enums
@@ -304,10 +305,8 @@ class Client(object):
         params.SecurityMode = self.security_policy.Mode
         params.RequestedLifetime = self.secure_channel_timeout
         # length should be equal to the length of key of symmetric encryption
-        nonce = utils.create_nonce(self.security_policy.symmetric_key_size)
-        params.ClientNonce = nonce  # this nonce is used to create a symmetric key
+        params.ClientNonce = utils.create_nonce(self.security_policy.symmetric_key_size) # this nonce is used to create a symmetric key
         result = self.uaclient.open_secure_channel(params)
-        self.security_policy.make_symmetric_key(nonce, result.ServerNonce)
         self.secure_channel_timeout = result.SecurityToken.RevisedLifetime
 
     def close_secure_channel(self):
@@ -570,3 +569,45 @@ class Client(object):
         This enums will be available in ua module
         """
         return load_enums(self)
+
+    def register_nodes(self, nodes):
+        """
+        Register nodes for faster read and write access (if supported by server)
+        Rmw: This call modifies the nodeid of the nodes, the original nodeid is
+        available as node.basenodeid
+        """
+        nodeids = [node.nodeid for node in nodes]
+        nodeids = self.uaclient.register_nodes(nodeids)
+        for node, nodeid in zip(nodes, nodeids):
+            node.basenodeid = node.nodeid
+            node.nodeid = nodeid
+        return nodes
+
+    def unregister_nodes(self, nodes):
+        """
+        Unregister nodes
+        """
+        nodeids = [node.nodeid for node in nodes]
+        self.uaclient.unregister_nodes(nodeids)
+        for node in nodes:
+            node.nodeid = node.basenodeid
+            node.basenodeid = None
+
+    def get_values(self, nodes):
+        """
+        Read the value of multiple nodes in one roundtrip.
+        """
+        nodes = [node.nodeid for node in nodes]
+        results = self.uaclient.get_attributes(nodes, ua.AttributeIds.Value)
+        return [result.Value.Value for result in results]
+
+    def set_values(self, nodes, values):
+        """
+        Write values to multiple nodes in one ua call
+        """
+        nodeids = [node.nodeid for node in nodes]
+        dvs = [ua_utils.value_to_datavalue(val) for val in values]
+        results = self.uaclient.set_attributes(nodeids, dvs, ua.AttributeIds.Value)
+        for result in results:
+            result.check()
+
