@@ -51,6 +51,7 @@ class Node(object):
             self.nodeid = ua.NodeId(nodeid, 0)
         else:
             raise ua.UaError("argument to node must be a NodeId object or a string defining a nodeid found {0} of type {1}".format(nodeid, type(nodeid)))
+        self.basenodeid = None
 
     def __eq__(self, other):
         if isinstance(other, Node) and self.nodeid == other.nodeid:
@@ -665,7 +666,7 @@ class Node(object):
         return opcua.common.manage_nodes.create_variable_type(self, nodeid, bname, datatype)
 
     def add_data_type(self, nodeid, bname, description=None):
-        return opcua.common.manage_nodes.create_data_type(self, nodeid, bname, description=None)
+        return opcua.common.manage_nodes.create_data_type(self, nodeid, bname, description=description)
 
     def add_property(self, nodeid, bname, val, varianttype=None, datatype=None):
         return opcua.common.manage_nodes.create_property(self, nodeid, bname, val, varianttype, datatype)
@@ -678,3 +679,20 @@ class Node(object):
 
     def call_method(self, methodid, *args):
         return opcua.common.methods.call_method(self, methodid, *args)
+
+    def register(self):
+        """
+        Register node for faster read and write access (if supported by server)
+        Rmw: This call modifies the nodeid of the node, the original nodeid is
+        available as node.basenodeid
+        """
+        nodeid = self.server.register_nodes([self.nodeid])[0]
+        self.basenodeid = self.nodeid
+        self.nodeid = nodeid
+
+    def unregister(self):
+        if self.basenodeid is None:
+            return
+        self.server.unregister_nodes([self.nodeid])
+        self.nodeid = self.basenodeid
+        self.basenodeid = None
