@@ -216,11 +216,9 @@ class StatusCode(FrozenClass):
 
     def __init__(self, value=0):
         if isinstance(value, str):
-            self.name = value
             self.value = getattr(status_codes.StatusCodes, value)
         else:
             self.value = value
-            self.name, self.doc = status_codes.get_name_and_doc(value)
         self._freeze = True
 
     def check(self):
@@ -242,6 +240,16 @@ class StatusCode(FrozenClass):
         else:
             return True
 
+    @property
+    def name(self):
+        name, _ = status_codes.get_name_and_doc(self.value)
+        return name
+
+    @property
+    def doc(self):
+        _, doc = status_codes.get_name_and_doc(self.value)
+        return doc
+
     def __str__(self):
         return 'StatusCode({0})'.format(self.name)
 
@@ -764,10 +772,22 @@ class Variant(FrozenClass):
     def _guess_type(self, val):
         if isinstance(val, (list, tuple)):
             error_val = val
-        while isinstance(val, (list, tuple)):
-            if len(val) == 0:
-                raise UaError("could not guess UA type of variable {0}".format(error_val))
-            val = val[0]
+            while val and isinstance(val[0], (list, tuple)):
+                val = val[0]
+
+            types = {type(el) for el in val}
+            if len(types) == 0:
+                raise UaError("List of zero length. Could not guess UA type of variable {0}".format(error_val))
+            elif types == set([int, float]):
+                logger.debug(
+                    "Variable {0} has ints and floats. UA type will be {1}".format(error_val, VariantType.Double)
+                )
+                val = float()
+            elif len(types) > 1:
+                raise UaError("List of multiple types. Could not guess UA type of variable {0}".format(error_val))
+            else:
+                val = val[0]
+
         if val is None:
             return VariantType.Null
         elif isinstance(val, bool):
