@@ -521,8 +521,10 @@ class CommonTests(object):
         self.assertEqual([1], val)
 
     def test_use_namespace(self):
-        idx = self.opc.get_namespace_index("urn:freeopcua:python:server")
-        self.assertEqual(idx, 1)
+        idx_orig = 1
+        uri = self.opc.get_namespace_array()[idx_orig]
+        idx = self.opc.get_namespace_index(uri)
+        self.assertEqual(idx_orig, idx)
         root = self.opc.get_root_node()
         myvar = root.add_variable(idx, 'var_in_custom_namespace', [5])
         myid = myvar.nodeid
