@@ -20,6 +20,7 @@ from opcua.common.xmlimporter import XmlImporter
 from opcua.ua.uatypes import _MaskEnum
 from opcua.common.structures import StructGenerator
 from opcua.common.connection import MessageChunk
+from opcua.ua.uaerrors import UaError
 
 
 class TestUnit(unittest.TestCase):
@@ -218,6 +219,18 @@ class TestUnit(unittest.TestCase):
         self.assertEqual(string_to_val(s_statuscode, ua.VariantType.StatusCode), statuscode)
         self.assertEqual(string_to_val(s_statuscode2, ua.VariantType.StatusCode), statuscode2)
 
+    def test_status_code_to_string(self):
+        # serialize a status code and deserialize it, name and doc resolution should work just fine
+        statuscode = ua.StatusCode(ua.StatusCodes.BadNotConnected)
+        statuscode2 = struct_from_binary(ua.StatusCode, io.BytesIO(struct_to_binary(ua.StatusCode(ua.StatusCodes.BadNotConnected))))
+
+        self.assertEqual(statuscode, statuscode2)
+        self.assertEqual(statuscode.value, statuscode2.value)
+
+        # properties that are not serialized should still translate properly
+        self.assertEqual(statuscode.name, statuscode2.name)
+        self.assertEqual(statuscode.doc, statuscode2.doc)
+
     def test_string_to_variant_qname(self):
         string = "2:name"
         obj = ua.QualifiedName("name", 2)
@@ -500,7 +513,6 @@ class TestUnit(unittest.TestCase):
     def test_variant_array(self):
         v = ua.Variant([1, 2, 3, 4, 5])
         self.assertEqual(v.Value[1], 2)
-        # self.assertEqual(v.VarianType, ua.VariantType.Int64) # we do not care, we should aonly test for sutff that matter
         v2 = variant_from_binary(ua.utils.Buffer(variant_to_binary(v)))
         self.assertEqual(v.Value, v2.Value)
         self.assertEqual(v.VariantType, v2.VariantType)
@@ -513,6 +525,29 @@ class TestUnit(unittest.TestCase):
         self.assertEqual(v.Value, v2.Value)
         self.assertEqual(v.VariantType, v2.VariantType)
 
+    def test_variant_array_mixed_types(self):
+        v = ua.Variant([1, 2, 3])
+        self.assertEqual(v.VariantType, ua.VariantType.Int64)
+
+        # No empty lists
+        with self.assertRaises(UaError):
+            v = ua.Variant([])
+
+        # First list is used to guess type
+        with self.assertRaises(UaError):
+            v = ua.Variant([[], [1, 2, 3]])
+
+        v = ua.Variant([[1, 2, 3], []])
+        self.assertEqual(v.VariantType, ua.VariantType.Int64)
+
+        # No mixed lists
+        with self.assertRaises(UaError):
+            v = ua.Variant([1, 2.0, '3'])
+
+        # Unless it's a mixed list of ints and floats
+        v = ua.Variant([1, 2.0, 3])
+        self.assertEqual(v.VariantType, ua.VariantType.Double)
+
     def test_variant_array_dim(self):
         v = ua.Variant([1, 2, 3, 4, 5, 6], dimensions=[2, 3])
         self.assertEqual(v.Value[1], 2)
